@@ -4,7 +4,11 @@ import numpy as np
 import pytorch_lightning as pl
 from abc import ABC, abstractmethod
 import torch
-from typing import List, Dict
+from torch.optim.optimizer import Optimizer
+from typing import List, Dict, Optional, Callable, Union, Any
+from omegaconf import OmegaConf
+
+from torch.optim.lr_scheduler import LambdaLR
 
 from transformers import AdamW
 from transformers import get_linear_schedule_with_warmup
@@ -18,22 +22,27 @@ from nerblackbox.modules.ner_training.data_preprocessing.data_preprocessor impor
 )
 from nerblackbox.modules.utils.util_functions import split_parameters
 from nerblackbox.modules.ner_training.ner_model_evaluation import NerModelEvaluation
+from nerblackbox.modules.ner_training.logging.mlflow_client import MLflowClient
+from nerblackbox.modules.ner_training.logging.default_logger import DefaultLogger
 
 NEWLINE_TOKENS = ["[newline]", "[NEWLINE]"]
 
 
 class NerModel(pl.LightningModule, ABC):
-    def __init__(self, hparams):
+    def __init__(self, hparams: OmegaConf):
         """
-        :param hparams: [argparse.Namespace] attr: experiment_name, run_name, pretrained_model_name, dataset_name, ..
+        :param hparams: attr: experiment_name, run_name, pretrained_model_name, dataset_name, ..
         """
         super().__init__()
-        self.hparams = hparams
+        self.save_hyperparameters(hparams)
 
         # split up hparams
-        self.params, self._hparams, self.log_dirs, self.experiment = split_parameters(
-            hparams
-        )
+        (
+            self.params,
+            self.hyperparameters,
+            self.log_dirs,
+            self.experiment,
+        ) = split_parameters(hparams)
 
         # preparations
         self._preparations()
@@ -61,7 +70,10 @@ class NerModel(pl.LightningModule, ABC):
         :created attr: scheduler              [torch LambdaLR]
         :return: -
         """
-        pass
+        self.mlflow_client: MLflowClient
+        self.default_logger: DefaultLogger
+        self.scheduler: LambdaLR
+        self.tag_list: List[str]
 
     def _preparations_data_general(self):
         """
@@ -97,7 +109,7 @@ class NerModel(pl.LightningModule, ABC):
         self.data_preprocessor = DataPreprocessor(
             tokenizer=self.tokenizer,
             do_lower_case=self.params.uncased,  # can be True !!
-            max_seq_length=self._hparams.max_seq_length,
+            max_seq_length=self.hyperparameters.max_seq_length,
             default_logger=self.default_logger,
         )
 
@@ -158,11 +170,21 @@ class NerModel(pl.LightningModule, ABC):
         return self.optimizer
 
     def optimizer_step(
-        self, current_epoch, batch_nb, optimizer, optimizer_i, second_order_closure=None
-    ):
+        self,
+        epoch: int = None,
+        batch_idx: int = None,
+        optimizer: Optimizer = None,
+        optimizer_idx: int = None,
+        optimizer_closure: Optional[Callable] = None,
+        on_tpu: bool = None,
+        using_native_amp: bool = None,
+        using_lbfgs: bool = None,
+    ) -> None:
+
         # update params
-        optimizer.step()
-        optimizer.zero_grad()
+        if optimizer is not None:
+            optimizer.step(closure=optimizer_closure)
+            optimizer.zero_grad()
 
         # update learning rate
         self.scheduler.step()
@@ -292,7 +314,9 @@ class NerModel(pl.LightningModule, ABC):
         # optimizer = BertAdam(optimizer_grouped_parameters,lr=2e-5, warmup=.1)
         return optimizer
 
-    def _create_scheduler(self, _lr_warmup_epochs, _lr_schedule, _lr_num_cycles=None):
+    def _create_scheduler(
+        self, _lr_warmup_epochs, _lr_schedule, _lr_num_cycles=None
+    ) -> LambdaLR:
         """
         create scheduler with warmup
         ----------------------------
@@ -309,7 +333,7 @@ class NerModel(pl.LightningModule, ABC):
         ]:
             raise Exception(f"lr_schedule = {_lr_schedule} not implemented.")
 
-        num_training_steps = self._get_steps(self._hparams.max_epochs)
+        num_training_steps = self._get_steps(self.hyperparameters.max_epochs)
         num_warmup_steps = self._get_steps(_lr_warmup_epochs)
 
         scheduler_params = {
@@ -358,7 +382,7 @@ class NerModel(pl.LightningModule, ABC):
         return _num_epochs * len(self.dataloader["train"])
 
     def _validate_on_epoch(
-        self, phase: str, outputs: List[List[torch.Tensor]]
+        self, phase: str, outputs: List[Union[torch.Tensor, Dict[str, Any]]]
     ) -> Dict[str, float]:
         """
         Args:
@@ -371,26 +395,41 @@ class NerModel(pl.LightningModule, ABC):
         ner_model_evaluation = NerModelEvaluation(
             current_epoch=self.current_epoch,
             tag_list=self.tag_list,
-            dataset_tags=self.params.dataset_tags,
+            annotation_scheme=self.params.dataset_tags
+            if self.params.dataset_tags != "iob"
+            else "bio",  # TODO: clean
             default_logger=self.default_logger,
             logged_metrics=self.logged_metrics,
         )
-        epoch_metrics, classification_report, epoch_loss = ner_model_evaluation.execute(
-            phase, outputs
-        )
-        self._log_metrics_and_classification_report(
-            phase, epoch_metrics, classification_report
+        (
+            epoch_metrics,
+            classification_report,
+            confusion_matrix,
+            epoch_loss,
+        ) = ner_model_evaluation.execute(phase, outputs)
+        self._log_metrics_confusion_matrix_classification_report(
+            phase,
+            epoch_metrics,
+            confusion_matrix,
+            classification_report,
         )
+        self.log(f"{phase}_loss", epoch_loss)  # for early stopping callback
+
         return {f"{phase}_loss": epoch_loss}
 
-    def _log_metrics_and_classification_report(
-        self, phase: str, epoch_metrics: Dict[str, np.array], classification_report: str
+    def _log_metrics_confusion_matrix_classification_report(
+        self,
+        phase: str,
+        epoch_metrics: Dict[str, np.array],
+        confusion_matrix: Optional[str] = "",
+        classification_report: Optional[str] = "",
     ) -> None:
         """
         Args:
-            phase:                 [str] 'val', 'test'
-            epoch_metrics          [dict] w/ keys 'all_acc', 'fil_f1_micro', .. & values = [np array]
-            classification_report: [str]
+            phase:                  'val', 'test'
+            epoch_metrics           keys 'all_acc', 'fil_f1_micro', .. & values = [np array]
+            confusion_matrix:
+            classification_report:
         """
         # tracked metrics & classification reports
         self._add_epoch_metrics(
@@ -401,13 +440,21 @@ class NerModel(pl.LightningModule, ABC):
         self._write_metrics_for_tensorboard(phase, epoch_metrics)
 
         # logging: mlflow
-        if phase == "val":
+        if phase == "test":
             self.mlflow_client.log_metrics(self.current_epoch, epoch_metrics)
-        self.mlflow_client.log_classification_report(
-            classification_report,
-            overwrite=(phase == "val" and self.current_epoch == 0),
-        )
-        self.mlflow_client.finish_artifact_mlflow()
+
+            if confusion_matrix is not None:
+                self.mlflow_client.log_artifact(
+                    confusion_matrix,
+                    overwrite=True,
+                )
+            if classification_report is not None:
+                self.mlflow_client.log_artifact(
+                    classification_report,
+                    overwrite=False,
+                )
+
+            self.mlflow_client.finish_artifact_mlflow()
 
         # print
         self._print_metrics(phase, epoch_metrics, classification_report)
@@ -436,7 +483,7 @@ class NerModel(pl.LightningModule, ABC):
         """
         # tb_logs: all
         tb_logs = {
-            f'{phase}/{k.split("_", 1)[0].replace("+", "P")}/{k.split("_", 1)[1]}': v
+            f'{phase}/{k.split("_", 1)[0]}/{k.split("_", 1)[1]}': v
             for k, v in metrics.items()
         }
 
@@ -476,7 +523,7 @@ class NerModel(pl.LightningModule, ABC):
             f"tag_ids        shape|1st row: {_tag_ids.shape} | {_tag_ids[0]}"
         )
 
-    def _print_metrics(self, phase, _metrics, _classification_reports=None):
+    def _print_metrics(self, phase, _metrics, _classification_reports: Optional[str]):
         """
         :param phase:         [str] 'train' or 'val'
         :param _metrics:
@@ -488,31 +535,19 @@ class NerModel(pl.LightningModule, ABC):
             f"--- Epoch #{self.current_epoch} {phase.ljust(5).upper()} ----"
         )
         self.default_logger.log_info(
-            "all loss:         {:.2f}".format(_metrics["all_loss"])
+            "token  all loss:         {:.2f}".format(_metrics["token_all_loss"])
         )
         self.default_logger.log_info(
-            "all acc:          {:.2f}".format(_metrics["all_acc"])
-        )
-        self.default_logger.log_debug(
-            "all f1 (micro):   {:.2f}".format(_metrics["all_f1_micro"])
-        )
-        self.default_logger.log_debug(
-            "all f1 (macro):   {:.2f}".format(_metrics["all_f1_macro"])
+            "token  all acc:          {:.2f}".format(_metrics["token_all_acc"])
         )
         self.default_logger.log_info(
-            "all  f1 (micro):   {:.2f}".format(_metrics["all_f1_micro"])
-        )
-        self.default_logger.log_debug(
-            "all  f1 (macro):   {:.2f}".format(_metrics["all_f1_macro"])
+            "token  all f1 (micro):   {:.2f}".format(_metrics["token_all_f1_micro"])
         )
         self.default_logger.log_info(
-            "fil  f1 (micro):   {:.2f}".format(_metrics["fil_f1_micro"])
-        )
-        self.default_logger.log_debug(
-            "fil  f1 (macro):   {:.2f}".format(_metrics["fil_f1_macro"])
+            "token  fil f1 (micro):   {:.2f}".format(_metrics["token_fil_f1_micro"])
         )
         self.default_logger.log_info(
-            "chk  f1 (micro):   {:.2f}".format(_metrics["chk_f1_micro"])
+            "entity fil f1 (micro):   {:.2f}".format(_metrics["entity_fil_f1_micro"])
         )
         self.default_logger.log_info(f"-----------------------")
         if _classification_reports is not None:
