@@ -1,13 +1,21 @@
 import warnings
 import numpy as np
+import pandas as pd
 import torch
 from seqeval.metrics import classification_report as classification_report_seqeval
 from sklearn.metrics import classification_report as classification_report_sklearn
-from typing import List, Dict, Union, Tuple
+from sklearn.metrics import confusion_matrix as confusion_matrix_sklearn
+from typing import List, Dict, Union, Tuple, Any, Optional
 
 from nerblackbox.modules.ner_training.metrics.ner_metrics import NerMetrics
-from nerblackbox.modules.ner_training.metrics.ner_metrics import convert_to_chunk
-from nerblackbox.modules.utils.env_variable import env_variable
+from nerblackbox.modules.ner_training.metrics.ner_metrics import (
+    convert2bio,
+    convert2plain,
+)
+from nerblackbox.modules.ner_training.data_preprocessing.data_preprocessor import (
+    order_tag_list,
+    convert_tag_list_bio2plain,
+)
 
 
 class NerModelEvaluation:
@@ -15,13 +23,22 @@ class NerModelEvaluation:
         self,
         current_epoch: int,
         tag_list: List[str],
-        dataset_tags: List[str],
+        annotation_scheme: str,
         default_logger,
         logged_metrics,
     ):
+        """
+        Args:
+            current_epoch:     e.g. 1
+            tag_list:          e.g. ["O", "PER", "ORG"]
+            annotation_scheme: e.g. "plain", "bio"
+            default_logger:
+            logged_metrics:
+        """
         self.current_epoch = current_epoch
-        self.tag_list = tag_list
-        self.dataset_tags = dataset_tags
+        self.tag_list = order_tag_list(tag_list)
+        self.tag_list_plain = convert_tag_list_bio2plain(tag_list)
+        self.annotation_scheme = annotation_scheme
         self.default_logger = default_logger
         self.logged_metrics = logged_metrics
 
@@ -29,8 +46,8 @@ class NerModelEvaluation:
     # 2. VALIDATE / COMPUTE METRICS
     ####################################################################################################################
     def execute(
-        self, phase: str, outputs: List[List[torch.Tensor]]
-    ) -> Tuple[Dict[str, np.array], str, float]:
+        self, phase: str, outputs: List[Union[torch.Tensor, Dict[str, Any]]]
+    ) -> Tuple[Dict[str, np.array], str, str, float]:
         """
         - validate on all batches of one epoch, i.e. whole val or test dataset
 
@@ -41,6 +58,7 @@ class NerModelEvaluation:
         Returns:
             epoch_metrics          [dict] w/ keys 'all_acc', 'fil_f1_micro', .. & values = [np array]
             classification_report: [str]
+            confusion_matrix:      [str]
             epoch_loss:            [float] mean of of all batch losses
         """
         print()
@@ -53,15 +71,26 @@ class NerModelEvaluation:
         epoch_metrics, epoch_tags = self._compute_metrics(phase, np_epoch)
 
         # classification report
-        classification_report = self._get_classification_report(
-            phase, self.current_epoch, epoch_tags
-        )
+        if phase == "test":
+            confusion_matrix = self._get_confusion_matrix_str(
+                epoch_tags,
+                phase=phase,
+                epoch=self.current_epoch,
+            )
+            classification_report = self._get_classification_report(
+                epoch_tags,
+                phase=None,
+                epoch=None,
+            )
+        else:
+            classification_report = ""
+            confusion_matrix = ""
 
-        return epoch_metrics, classification_report, np_epoch["loss"]
+        return epoch_metrics, classification_report, confusion_matrix, np_epoch["loss"]
 
     @staticmethod
     def _convert_output_to_np_batch(
-        outputs: List[List[torch.Tensor]],
+        outputs: List[Union[torch.Tensor, Dict[str, Any]]],
     ) -> Dict[str, List[np.array]]:
         """
         - converts pytorch lightning output to np_batch dictionary
@@ -128,7 +157,7 @@ class NerModelEvaluation:
             _np_epoch: [dict] w/ key-value pairs:
                                  'loss':     [np value]
                                  'tag_ids':  [1D np array] of length      <batch_size> x <seq_length>
-                                 'logits'    [2D np array] of size shape [<batch_size> x <seq_length>, <num_tags>]
+                                 'logits':   [2D np array] of size shape [<batch_size> x <seq_length>, <num_tags>]
 
         Returns:
             _epoch_metrics  [dict] w/ keys 'all_acc', 'fil_f1_micro', .. & values = [np array]
@@ -154,17 +183,12 @@ class NerModelEvaluation:
             "pred:", np.shape(tags["pred"]), list(set(tags["pred"]))
         )
 
-        if phase == "val":
-            for field in ["true", "pred"]:
-                np.save(f'{env_variable("DIR_RESULTS")}/{field}.npy', tags[field])
-
         # batch / dataset metrics
-        _epoch_metrics = {"all_loss": _np_epoch["loss"]}
+        _epoch_metrics = {"token_all_loss": _np_epoch["loss"]}
         for tag_subset in [
             "all",
             "fil",
-            "chk",
-        ] + self.tag_list:
+        ] + self.tag_list_plain:
             _epoch_metrics.update(
                 self._compute_metrics_for_tags_subset(
                     _epoch_tags, phase, tag_subset=tag_subset
@@ -190,13 +214,14 @@ class NerModelEvaluation:
             pred_flat: [np array] of shape [batch_size * seq_length], _np_logits    reduced and flattened
         """
         true_flat = _np_tag_ids.flatten()
-        pred_flat = np.argmax(_np_logits, axis=2).flatten()
+        pred_flat = np.argmax(_np_logits, axis=-1).flatten()
         return true_flat, pred_flat
 
     def _convert_tag_ids_to_tags(self, _tag_ids: np.array) -> np.array:
         """
         helper method for _compute_metrics()
         convert tag_ids (int) to tags (str)
+        special tags [*] have tag_id = -100 and are converted to [S]
 
         Args:
             _tag_ids: [np array] of shape [batch_size * seq_length] with [int] elements
@@ -205,7 +230,10 @@ class NerModelEvaluation:
             _tags:    [np array] of shape [batch_size * seq_length] with [str] elements
         """
         return np.array(
-            [self.tag_list[tag_id] if tag_id >= 0 else "[S]" for tag_id in _tag_ids]
+            [
+                self.tag_list[int(tag_id)] if tag_id >= 0 else "[S]"
+                for tag_id in _tag_ids
+            ]
         )
 
     @staticmethod
@@ -242,113 +270,177 @@ class NerModelEvaluation:
         Returns:
             _metrics    [dict] w/ keys = metric (e.g. 'all_precision_micro') and value = [float]
         """
-        tag_list = self._get_filtered_tags(tag_subset)
-        tag_group = [tag_subset] if tag_subset in ["all", "fil", "chk"] else ["ind"]
-        level = "chunk" if tag_subset == "chk" else "token"
+        _tags_plain = {
+            field: convert2plain(
+                _tags[field], convert_to_plain=self.annotation_scheme != "plain"
+            )
+            for field in ["true", "pred"]
+        }
 
-        ner_metrics = NerMetrics(
-            _tags["true"],
-            _tags["pred"],
-            tag_list=tag_list,
-            level=level,
-            plain_tags=self.dataset_tags == "plain",
+        tag_list, tag_list_indices = self._get_filtered_tags(tag_subset, _tags_plain)
+        required_tag_groups = (
+            [tag_subset] if tag_subset in ["all", "fil", "O"] else ["ind"]
         )
-        ner_metrics.compute(
-            self.logged_metrics.get_metrics(tag_group=tag_group, phase_group=[_phase])
-        )
-        results = ner_metrics.results_as_dict()
+        required_phases = [_phase]
+
+        if tag_subset == "O":
+            levels = ["token"]
+        else:
+            levels = ["token", "entity"]
 
         _metrics = dict()
-        # simple
-        for metric_type in self.logged_metrics.get_metrics(
-            tag_group=tag_group,
-            phase_group=[_phase],
-            micro_macro_group=["simple"],
-            exclude=["loss"],
-        ):
-            # if results[metric_type] is not None:
-            _metrics[f"{tag_subset}_{metric_type}"] = results[metric_type]
+        for level in levels:
+            required_levels = [level]
+            metrics_to_compute = self.logged_metrics.get_metrics(
+                required_tag_groups=required_tag_groups,
+                required_phases=required_phases,
+                required_levels=required_levels,
+            )
 
-        # micro
-        for metric_type in self.logged_metrics.get_metrics(
-            tag_group=tag_group, phase_group=[_phase], micro_macro_group=["micro"]
-        ):
-            # if results[f'{metric_type}_micro'] is not None:
-            if tag_group == ["ind"]:
-                _metrics[f"{tag_subset}_{metric_type}"] = results[
-                    f"{metric_type}_micro"
-                ]
+            if len(metrics_to_compute):
+                ner_metrics = NerMetrics(
+                    _tags["true"] if level == "entity" else _tags_plain["true"],
+                    _tags["pred"] if level == "entity" else _tags_plain["pred"],
+                    tag_list=tag_list if level == "token" else None,
+                    tag_index=tag_list_indices if level == "entity" else None,
+                    level=level,
+                    plain_tags=self.annotation_scheme == "plain"
+                    if level == "entity"
+                    else True,
+                )
+                ner_metrics.compute(metrics_to_compute)
+                results = ner_metrics.results_as_dict()
             else:
-                _metrics[f"{tag_subset}_{metric_type}_micro"] = results[
-                    f"{metric_type}_micro"
-                ]
+                results = dict()
 
-        # macro
-        for metric_type in self.logged_metrics.get_metrics(
-            tag_group=tag_group, phase_group=[_phase], micro_macro_group=["macro"]
-        ):
-            # if results[f'{metric_type}_macro'] is not None:
-            _metrics[f"{tag_subset}_{metric_type}_macro"] = results[
-                f"{metric_type}_macro"
-            ]
+            # simple
+            for metric_type in self.logged_metrics.get_metrics(
+                required_tag_groups=required_tag_groups,
+                required_phases=required_phases,
+                required_levels=required_levels,
+                required_averaging_groups=["simple"],
+                exclude=["numberofclasses", "loss"],
+            ):
+                _metrics[f"{level}_{tag_subset}_{metric_type}"] = results[metric_type]
+
+            # micro
+            for metric_type in self.logged_metrics.get_metrics(
+                required_tag_groups=required_tag_groups,
+                required_phases=required_phases,
+                required_levels=required_levels,
+                required_averaging_groups=["micro"],
+                exclude=["numberofclasses"],
+            ):
+                if required_tag_groups in [["O"], ["ind"]]:
+                    _metrics[f"{level}_{tag_subset}_{metric_type}"] = results[
+                        f"{metric_type}_micro"
+                    ]
+                else:
+                    _metrics[f"{level}_{tag_subset}_{metric_type}_micro"] = results[
+                        f"{metric_type}_micro"
+                    ]
+
+            # macro
+            for metric_type in self.logged_metrics.get_metrics(
+                required_tag_groups=required_tag_groups,
+                required_phases=required_phases,
+                required_levels=required_levels,
+                required_averaging_groups=["macro"],
+            ):
+                _metrics[f"{level}_{tag_subset}_{metric_type}_macro"] = results[
+                    f"{metric_type}_macro"
+                ]
 
         return _metrics
 
-    def _get_filtered_tags(self, _tag_subset: str) -> List[str]:
+    def _get_filtered_tags(
+        self, _tag_subset: str, _tags_plain: Optional[Dict[str, np.array]] = None
+    ) -> Tuple[List[str], Optional[int]]:
         """
         helper method for _compute_metrics()
         get list of filtered tags corresponding to _tag_subset name
 
         Args:
             _tag_subset: [str], e.g. 'all', 'fil', 'PER'
+            _tags_plain: [dict] w/ keys 'true', 'pred'      & values = [np array]
 
         Returns:
-            _filtered_tags: [list] of filtered tags [str]
+            _filtered_tags:       list of filtered tags
+            _filtered_tags_index: filtered tags index in case of single _filtered_tag, ignoring "O"
         """
         if _tag_subset == "all":
-            return self.tag_list
-        elif _tag_subset in ["fil", "chk"]:
-            return [tag for tag in self.tag_list if tag != "O"]
+            _filtered_tags = self.tag_list_plain
+            _filtered_tags_index = None
+        elif _tag_subset == "fil":
+            _filtered_tags = [tag for tag in self.tag_list_plain if tag != "O"]
+            _filtered_tags_index = None
         else:
-            return [_tag_subset]
+            assert _tags_plain is not None, f"ERROR! need to provide _tags_plain"
+            tag_list_plain_filtered = [
+                elem
+                for elem in self.tag_list_plain
+                if (elem in _tags_plain["true"] or elem in _tags_plain["pred"])
+                and elem != "O"
+            ]
+            _filtered_tags = [_tag_subset]
+
+            try:
+                _filtered_tags_index_list = [tag_list_plain_filtered.index(_tag_subset)]
+                assert len(_filtered_tags_index_list) == 1
+                _filtered_tags_index = _filtered_tags_index_list[0]
+            except ValueError:
+                _filtered_tags_index = None
+
+        return _filtered_tags, _filtered_tags_index
 
     ####################################################################################################################
     # 2. CLASSIFICATION REPORT #########################################################################################
     ####################################################################################################################
     def _get_classification_report(
-        self, phase: str, epoch: int, epoch_tags: Dict[str, np.array]
+        self,
+        epoch_tags: Dict[str, np.array],
+        phase: Optional[str] = None,
+        epoch: Optional[int] = None,
     ) -> str:
         """
         - get token-based (sklearn) & chunk-based (seqeval) classification report
 
         Args:
+            epoch_tags:     [dict] w/ keys 'true', 'pred'      & values = [np array]
             phase:          [str], 'train', 'val', 'test'
             epoch:          [int]
-            epoch_tags:     [dict] w/ keys 'true', 'pred'      & values = [np array]
 
         Returns:
             classification_report: [str]
         """
         warnings.filterwarnings("ignore")
 
-        # token-based classification report
-        tag_list_filtered = self._get_filtered_tags("fil")
+        epoch_tags_plain = {
+            field: convert2plain(
+                epoch_tags[field], convert_to_plain=self.annotation_scheme != "plain"
+            )
+            for field in ["true", "pred"]
+        }
+
+        # token-based classification report, plain tags
+        tag_list_filtered, _ = self._get_filtered_tags("fil")
         classification_report: str = ""
-        classification_report += f"\n>>> Phase: {phase} | Epoch: {epoch}"
+        if phase is not None and epoch is not None:
+            classification_report += f"\n>>> Phase: {phase} | Epoch: {epoch}"
         classification_report += (
-            "\n--- token-based (sklearn) classification report on fil ---\n"
+            "\n--- token-based, plain tag (sklearn) classification report on fil ---\n"
         )
         classification_report += classification_report_sklearn(
-            epoch_tags["true"], epoch_tags["pred"], labels=tag_list_filtered
+            epoch_tags_plain["true"], epoch_tags_plain["pred"], labels=tag_list_filtered
         )
 
         # chunk-based classification report
         epoch_tags_chunk = dict()
         for field in ["true", "pred"]:
-            epoch_tags_chunk[field] = convert_to_chunk(
-                epoch_tags[field], to_bio=self.dataset_tags == "plain"
+            epoch_tags_chunk[field] = convert2bio(
+                epoch_tags[field], convert_to_bio=self.annotation_scheme == "plain"
             )
-        self.default_logger.log_debug("> dataset_tags:", self.dataset_tags)
+        self.default_logger.log_debug("> annotation_scheme:", self.annotation_scheme)
         self.default_logger.log_debug(
             "> epoch_tags_chunk[true]:", list(set(epoch_tags_chunk["true"]))
         )
@@ -357,12 +449,60 @@ class NerModelEvaluation:
         )
 
         classification_report += (
-            "\n--- chunk-based (seqeval) classification report on fil ---\n"
+            "\n--- entity-based (seqeval) classification report on fil ---\n"
         )
         classification_report += classification_report_seqeval(
-            epoch_tags_chunk["true"], epoch_tags_chunk["pred"], suffix=False
+            [epoch_tags_chunk["true"]], [epoch_tags_chunk["pred"]], suffix=False
         )
 
         warnings.resetwarnings()
 
         return classification_report
+
+    def _get_confusion_matrix_str(
+        self,
+        epoch_tags: Dict[str, np.array],
+        phase: Optional[str] = None,
+        epoch: Optional[int] = None,
+    ) -> str:
+        """
+        - get token-based (sklearn) confusion matrix
+
+        Args:
+            epoch_tags:     [dict] w/ keys 'true', 'pred'      & values = [np array]
+            phase:          [str], 'train', 'val', 'test'
+            epoch:          [int]
+
+        Returns:
+            confusion_matrix_str:  [str] with confusion matrix as pd dataframe
+        """
+        warnings.filterwarnings("ignore")
+
+        epoch_tags_plain = {
+            field: convert2plain(
+                epoch_tags[field], convert_to_plain=self.annotation_scheme != "plain"
+            )
+            for field in ["true", "pred"]
+        }
+
+        # token-based confusion matrix, plain tags
+        confusion_matrix = confusion_matrix_sklearn(
+            epoch_tags_plain["true"],
+            epoch_tags_plain["pred"],
+            labels=self.tag_list_plain,
+        )
+
+        df_confusion_matrix = pd.DataFrame(confusion_matrix)
+        df_confusion_matrix.columns = self.tag_list_plain
+        df_confusion_matrix.index = self.tag_list_plain
+
+        confusion_matrix_str: str = ""
+        if phase is not None and epoch is not None:
+            confusion_matrix_str += f"\n>>> Phase: {phase} | Epoch: {epoch}\n"
+        confusion_matrix_str += (
+            "\n--- token-based, plain tag (sklearn) confusion matrix on all ---"
+            + "\n... rows = ground truth | columns = predictions\n"
+            + f"{df_confusion_matrix.to_string()}"
+        )
+
+        return confusion_matrix_str
