@@ -17,6 +17,8 @@ from nerblackbox.modules.ner_training.data_preprocessing.tools.bert_dataset impo
 )
 from nerblackbox.modules.ner_training.data_preprocessing.data_preprocessor import (
     DataPreprocessor,
+    order_tag_list,
+    convert_tag_list_bio2plain,
 )
 from nerblackbox.modules.ner_training.data_preprocessing.tools.utils import (
     EncodingsKeys,
@@ -47,6 +49,9 @@ data_preprocessor = DataPreprocessor(
 )
 
 
+########################################################################################################################
+########################################################################################################################
+########################################################################################################################
 class TestCsvReaderAndDataProcessor:
 
     ####################################################################################################################
@@ -175,6 +180,9 @@ class TestCsvReaderAndDataProcessor:
             # TODO: further testing
 
 
+########################################################################################################################
+########################################################################################################################
+########################################################################################################################
 class TestInputExamplesToTensorsAndBertDataset:
     @pytest.mark.parametrize(
         "texts, "
@@ -435,6 +443,9 @@ class TestInputExamplesToTensorsAndBertDataset:
                 ), f"{string} = {data[j][i]} != {true[j]}"
 
 
+########################################################################################################################
+########################################################################################################################
+########################################################################################################################
 class TestMisc:
 
     ####################################################################################################################
@@ -451,7 +462,7 @@ class TestMisc:
             ),
         ],
     )
-    def tests(
+    def test_ensure_completeness_in_case_of_bio_tags(
         self,
         tag_list: List[str],
         returned_tag_list: List[str],
@@ -465,8 +476,67 @@ class TestMisc:
             test_returned_tag_list == returned_tag_list
         ), f"test_returned_tag_list = {test_returned_tag_list} != {returned_tag_list}"
 
+    ####################################################################################################################
+    @pytest.mark.parametrize(
+        "tag_list, " "tag_list_ordered",
+        [
+            (
+                ["O", "PER", "ORG", "MISC"],
+                ["O", "MISC", "ORG", "PER"],
+            ),
+            (
+                ["PER", "ORG", "O", "MISC"],
+                ["O", "MISC", "ORG", "PER"],
+            ),
+            (
+                ["O", "B-PER", "I-MISC", "B-ORG", "I-PER", "B-MISC", "I-ORG"],
+                ["O", "B-MISC", "B-ORG", "B-PER", "I-MISC", "I-ORG", "I-PER"],
+            ),
+        ],
+    )
+    def test_order_tag_list(
+        self,
+        tag_list: List[str],
+        tag_list_ordered: List[str],
+    ) -> None:
+        test_tag_list_ordered = order_tag_list(tag_list)
+        assert (
+            test_tag_list_ordered == tag_list_ordered
+        ), f"test_tag_list_ordered = {test_tag_list_ordered} != {tag_list_ordered}"
+
+    ####################################################################################################################
+    @pytest.mark.parametrize(
+        "tag_list_bio, " "tag_list",
+        [
+            (
+                ["O", "B-MISC", "B-ORG", "B-PER", "I-MISC", "I-ORG", "I-PER"],
+                ["O", "MISC", "ORG", "PER"],
+            ),
+            (  # if applied to plain tag_list, nothing happens
+                ["O", "MISC", "ORG", "PER"],
+                ["O", "MISC", "ORG", "PER"],
+            ),
+        ],
+    )
+    def test_convert_tag_list_bio2plain(
+        self,
+        tag_list_bio: List[str],
+        tag_list: List[str],
+    ) -> None:
+        test_tag_list = convert_tag_list_bio2plain(tag_list_bio)
+        assert (
+            test_tag_list == tag_list
+        ), f"test_tag_list_ordered = {test_tag_list} != {tag_list}"
+
 
 if __name__ == "__main__":
-    TestCsvReaderAndDataProcessor().tests()
-    TestInputExamplesToTensorsAndBertDataset().tests()
-    TestMisc().tests()
+    test_csv = TestCsvReaderAndDataProcessor()
+    test_csv.tests()
+
+    test_input_examples_to_tensors = TestInputExamplesToTensorsAndBertDataset()
+    test_input_examples_to_tensors.tests()
+
+    test_misc = TestMisc()
+    test_misc.test_ensure_completeness_in_case_of_bio_tags()
+    test_misc.test_order_tag_list()
+    test_misc.test_convert_tag_list_bio2plain()
